/*
 * NRF.c
 *
 *  Created on: May 30, 2019
 *      Author: Tomas
 */
#include "NRF.h"

void initNRF() {
	RTC->BKP1R = STATE_NRF_ACTIVE;
	nRF52init();

}

void initCommandMode() {
    initNRF();
    startNRF( RECEIVE_COMMANDS );

	nrf_mode = NRF_COMMAND_MODE;
	nrf_start_time = HAL_GetTick();
	nrf_startup = 0;
}

void magneticTelemetry( Mag3110 * val ) {

	char buffer[ 32 ];


	buffer[0] = 0xAA;
	buffer[1] = 0xAB;

	for( uint8_t x = 0; x < 8; x++ )
		buffer[ 2 + x ] = un_device_id[ x ];

	buffer[ 10 ] = NRF_COMMAND_MAG_TELEMETRY;
	buffer[ 11 ] = 1;


	buffer[ 12 ] = (val->x >> 8) & 0xFF;
	buffer[ 13 ] = val->x & 0xFF;

	buffer[ 14 ] = (val->y >> 8) & 0xFF;
	buffer[ 15 ] = val->y & 0xFF;

	buffer[ 16 ] = (val->z >> 8) & 0xFF;
	buffer[ 17 ] = val->z & 0xFF;

	buffer[ 18 ] = (data_x.a >> 8) & 0xFF;
	buffer[ 19 ] = (data_x.a) & 0xFF;

	buffer[ 18 ] = (data_mag.a >> 8) & 0xFF;
	buffer[ 19 ] = (data_mag.a) & 0xFF;

	buffer[ 18 ] = (data_rotation.a >> 8) & 0xFF;
	buffer[ 19 ] = (data_rotation.a) & 0xFF;

	buffer[ 18 ] = (data_total.a >> 8) & 0xFF;
	buffer[ 19 ] = (data_total.a) & 0xFF;



	//buffer[ 26 ] = main_loop.park_state;

	nrfEmitData( buffer );

}

uint32_t getLairdNrfVersion() {
	char buffer[ 3 ];

	nRF52ReadRegister( 0x04, buffer, 3 );

	return ( buffer[0] << 16 ) + ( buffer[1] << 8 ) + ( buffer[2] );

}

uint8_t nrfSanityCheck( uint8_t test ) {

	#ifdef NRF52
		return nRF52SanityCheck( test );

	#else
		return NRF24L01_SanityCheck();

	#endif

}

void startNRF( uint8_t mode ) {
	stopNRFMode();
	HAL_Delay( 2 );
	initNRF();
	HAL_Delay( 1 );
	#ifdef NRF52

		nRF52SetIRQPin();
		nRF52StartReceiveMode( mode );
	#else
		startNRF( mode );
	#endif

}

void stopNRFMode() {
	HAL_SPI_DeInit( &hspi1 );

	HAL_GPIO_DeInit( GPIOA, GPIO_PIN_5 );
	HAL_GPIO_DeInit( GPIOA, GPIO_PIN_6 );
	HAL_GPIO_DeInit( GPIOA, GPIO_PIN_7 );

	HAL_GPIO_WritePin( NRF_CSN_Port, NRF_CSN_Pin, GPIO_PIN_RESET );
	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, GPIO_PIN_RESET );
	HAL_GPIO_WritePin( SPI_EN_Port, SPI_EN_Pin, GPIO_PIN_RESET );
	HAL_GPIO_WritePin( BOOST_EN_Port, BOOST_EN_Pin, GPIO_PIN_RESET );

}
void nrfEmitData( uint8_t * buffer ) {

	#ifdef NRF52
		//startNRF( NRF_COMMAND_TRANSMIT_MODE );
		stopNRFMode();
		HAL_Delay( 100 );
		initNRF();
		nR52Emit( buffer );
		HAL_Delay( 900 );
	#else
		startNRF( mode );
	#endif


}

uint8_t checkID( char * buffer_id ) {
	uint8_t ok = ID_MATCH;
	for( int x = 0; x < 8; x++ ) {
		 if ( buffer_id[ x ] == 0 ) {
			ok = ID_BROADCAST;
		 }else{
			 ok = ID_MISMATCH;
			 break;
		 }
	}

	if ( ok == ID_MISMATCH ) {
		for( int x = 0; x < ID_SIZE; x++ ) {
			 if ( buffer_id[ x ] != un_device_id[ x ] ) {
				 ok = ID_MISMATCH;
				 break;
			 }else{
				 ok = ID_MATCH;
			 }
		}
	}

	if ( ok == ID_MISMATCH ) {
		for( int x = 0; x < ID_SIZE; x++ ) {
			 if ( buffer_id[ x ] != NETWORK_ID_CONSTANT ) {
				 ok = ID_MISMATCH;
				 break;
			 }else{
				 ok = ID_MATCH;
			 }
		}
	}

	if ( ok == ID_MISMATCH )
		return 0;

	return ok;
}

uint8_t nRFCommandStep() {
	uint8_t n = 1;
	char buffer1[32];
	char buf[32];

	HAL_IWDG_Refresh( &hiwdg );

	if ( nrfSanityCheck( 7 ) == 0 ) {
		stopNRFMode();
		HAL_Delay( 500 );
		initNRF();
		startNRF( RECEIVE_COMMANDS );
	}

	while( nrfIsPacketWaiting() ) {


		if ( nrfGetPacket( buffer1 ) == 0 ) {
			return 0;
		}

		if (buffer1[ 0 ] == 0xAA && buffer1[ 1 ] == 0xAB ) {
			uint8_t command_code = buffer1[ 10 ];

			char buff_id[ 8 ];
			for( uint8_t x = 0; x < 8; x++ ) {
				buff_id[ x ] = buffer1[ x+2 ];
			}

			if ( checkID( buff_id ) == 0 ){
				return 0;
			}

			nrf_start_time = HAL_GetTick();

			if ( command_code == NRF_COMMAND_SET_TX_PWR ) {
				NRF_POWER = buffer1[ 11 ];

				buf[0] = 0xAA;
				buf[1] = 0xAB;
				for( uint8_t x = 0; x < ID_SIZE; x++ )
					buf[ 2 + x ] = un_device_id[ x ];
				buf[ 10 ] = NRF_COMMAND_SET_TX_PWR;
				buf[ 11 ] = 0x01;


				nrfEmitData( buf );
				startNRF( RECEIVE_COMMANDS );

				return NRF_COMMAND_SET_TX_PWR;
			}else if ( command_code == NRF_COMMAND_REBOOT ) {


				buf[0] = 0xAA;
				buf[1] = 0xAB;
				for( uint8_t x = 0; x < ID_SIZE; x++ )
					buf[ 2 + x ] = un_device_id[ x ];
				buf[ 10 ] = NRF_COMMAND_REBOOT;
				buf[ 11 ] = 0x01;
				RTC->BKP0R = 0;
				nrfEmitData( buf );

				startNRF( RECEIVE_COMMANDS );
				HAL_NVIC_SystemReset();
				return NRF_COMMAND_REBOOT;
			}else if ( command_code == NRF_COMMAND_SEND_MAC ) {

				buf[0] = 0xAA;
				buf[1] = 0xAB;
				for( uint8_t x = 0; x < ID_SIZE; x++ )
					buf[ 2 + x ] = un_device_id[ x ];
				buf[ 10 ] = NRF_COMMAND_SEND_MAC;
				buf[ 11 ] = 0x01;
				nrfEmitData( buf );

				radio_on();
				radio_send_msg( "61" );
				HAL_Delay( 5000 );
				radio_send_msg( "61" );
				HAL_Delay( 5000 );
				radio_off();

				startNRF( RECEIVE_COMMANDS );

				return NRF_COMMAND_SEND_MAC;
			}else if ( command_code == NRF_COMMAND_ENTER_BOOTLOADER ) {

				nrf_startup = 6;
				buf[0] = 0xAA;
				buf[1] = 0xAB;
				for( uint8_t x = 0; x < ID_SIZE; x++ )
					buf[ 2 + x ] = un_device_id[ x ];
				buf[ 10 ] = NRF_COMMAND_ENTER_BOOTLOADER;
				buf[ 11 ] = 0x01;

				nrfEmitData( buf );

				//stopNRFMode();
				return NRF_COMMAND_ENTER_BOOTLOADER;
			}else if ( command_code == NRF_COMMAND_ENTER_NRF_BURST_MODE ) {
				nrf_startup = 3;
				buf[0] = 0xAA;
				buf[1] = 0xAB;
				for( uint8_t x = 0; x < ID_SIZE; x++ )
					buf[ 2 + x ] = un_device_id[ x ];
				buf[ 10 ] = NRF_COMMAND_ENTER_NRF_BURST_MODE;
				buf[ 11 ] = 0x01;

				nrfEmitData( buf );

				startNRF( RECEIVE_COMMANDS );
				return NRF_COMMAND_ENTER_NRF_BURST_MODE;
			}else if ( command_code == NRF_COMMAND_EXIT_COMMAND_MODE ) {

				buf[0] = 0xAA;
				buf[1] = 0xAB;
				for( uint8_t x = 0; x < ID_SIZE; x++ )
					buf[ 2 + x ] = un_device_id[ x ];
				buf[ 10 ] = NRF_COMMAND_EXIT_COMMAND_MODE;
				buf[ 11 ] = 0x01;

				nrfEmitData( buf );
				//stopNRFMode();
				nrf_startup = 5;
				//startNRF( RECEIVE_COMMANDS );

				nrf_mode = NO_NRF_MODE;
				return NRF_COMMAND_EXIT_COMMAND_MODE;
			}else if ( command_code == NRF_COMMAND_ENTER_STORAGE_MODE ) {


				buf[0] = 0xAA;
				buf[1] = 0xAB;
				for( uint8_t x = 0; x < ID_SIZE; x++ )
					buf[ 2 + x ] = un_device_id[ x ];
				buf[ 10 ] = NRF_COMMAND_ENTER_STORAGE_MODE;
				buf[ 11 ] = 0x01;

				nrfEmitData( buf );

				nvmWriteOperatingMode( 0 );

				startNRF( RECEIVE_COMMANDS );
				return NRF_COMMAND_ENTER_STORAGE_MODE;
			}else if ( command_code == NRF_COMMAND_ANSWER_PING ) {
				buf[0] = 0xAA;
				buf[1] = 0xAB;
				for( uint8_t x = 0; x < ID_SIZE; x++ )
					buf[ 2 + x ] = un_device_id[ x ];
				buf[ 10 ] = NRF_COMMAND_ANSWER_PING;
				buf[ 11 ] = 0x01;

				nrfEmitData( buf );

				startNRF( RECEIVE_COMMANDS );
				return NRF_COMMAND_ANSWER_PING;
			}else if ( command_code == NRF_COMMAND_MAG_TELEMETRY ) {
				buf[0] = 0xAA;
				buf[1] = 0xAB;
				for( uint8_t x = 0; x < ID_SIZE; x++ )
					buf[ 2 + x ] = un_device_id[ x ];
				buf[ 10 ] = NRF_COMMAND_MAG_TELEMETRY;
				buf[ 11 ] = 0x01;
				nrf_mag_telemetry = 1;
				startup_time_mag_telemetry = HAL_GetTick();
				//emitData( EMIT_COMMANDS, buf, COMMAND_RESPONSE_TIME );

				return NRF_COMMAND_MAG_TELEMETRY;
			}else if ( command_code == NRF_COMMAND_SET_BASELINE ) {
				buf[0] = 0xAA;
				buf[1] = 0xAB;
				for( uint8_t x = 0; x < ID_SIZE; x++ )
					buf[ 2 + x ] = un_device_id[ x ];
				buf[ 10 ] = NRF_COMMAND_SET_BASELINE;
				buf[ 11 ] = 0x01;

				nrfEmitData( buf );


				data_x.b_old = ( buffer1[ 12 ] << 24 ) ;
				data_x.b_old += ( buffer1[ 13 ] << 16 );
				data_x.b_old += ( buffer1[ 14 ] << 8 );
				data_x.b_old += ( buffer1[ 15 ] );

				data_x.bellum = ( buffer1[ 12 ] << 24 );
				data_x.bellum += ( buffer1[ 13 ] << 16 );
				data_x.bellum += ( buffer1[ 14 ] << 8 );
				data_x.bellum += ( buffer1[ 15 ] );

				startNRF( RECEIVE_COMMANDS );
				return NRF_COMMAND_SET_BASELINE;



			}else if ( command_code == NRF_COMMAND_CHANGE_DR ) {
				buf[0] = 0xAA;
				buf[1] = 0xAB;
				for( uint8_t x = 0; x < ID_SIZE; x++ )
					buf[ 2 + x ] = un_device_id[ x ];
				buf[ 10 ] = NRF_COMMAND_SET_BASELINE;
				buf[ 11 ] = 0x01;

				nrfEmitData( buf );


				#ifdef MURATA
					lora_set_dr( buffer1[ 12 ] );
				#endif
				startNRF( RECEIVE_COMMANDS );


				return NRF_COMMAND_CHANGE_DR;
			}else if ( command_code == NRF_COMMAND_FLASH_LED ) {
				buf[0] = 0xAA;
				buf[1] = 0xAB;
				for( uint8_t x = 0; x < ID_SIZE; x++ )
					buf[ 2 + x ] = un_device_id[ x ];
				buf[ 10 ] = NRF_COMMAND_FLASH_LED;
				buf[ 11 ] = 0x01;

				for( uint8_t x = 0; x < 8; x++ ) {
					blinkLed( 150 );
					HAL_Delay( 150 );
				}

				nrfEmitData( buf );
				startNRF( RECEIVE_COMMANDS );
				return NRF_COMMAND_FLASH_LED;
			}else{

				char formated_buffer[ 22 ];

				for( uint8_t x = 0; x < 22; x++ ) {
					formated_buffer[ x ] = buffer1[ x + 10 ];
				}

				uint8_t recv1 = lora_recv_msg( formated_buffer );

				buf[0] = 0xAA;
				buf[1] = 0xAB;
				for( uint8_t x = 0; x < ID_SIZE; x++ )
					buf[ 2 + x ] = un_device_id[ x ];
				buf[ 10 ] = command_code;
				buf[ 11 ] = 0x01;

				nrfEmitData( buf );
				startNRF( RECEIVE_COMMANDS );
				return recv1;
			}

		}
		HAL_IWDG_Refresh( &hiwdg );

	}
	return 0;
}

uint8_t nrfIsPacketWaiting() {

	#ifdef NRF52
		return nRF52PacketReceived();

	#else
		if ( HAL_GPIO_ReadPin( NRF_IRQ_Port, NRF_IRQ_Pin )  ) {
			return 1;
		}else{

			uint8_t status = NRF24L01_ReadRegister( 0x07 );

			status = status & 0b00001110;
			status = status >> 1;

			if ( status == 0b111 )
				return 0;

			return 1;

		}
	#endif


}


uint8_t nrfGetPacket( char * buffer ) {

#ifdef NRF52
	return nRF52getPacket( buffer );
#else
	return nRF24ServiceResidentialParking( buffer );
#endif



}
