#include "NRF52.h"

uint8_t nRF52SanityCheck( uint8_t active_mode ) {

	uint8_t buffer[ 3 ];

	if ( nRF52ReadRegister( WHO_AM_I, buffer, 3 ) == 0 ) {
		return 0;
	}

	if ( buffer[ 0 ] == 0x01 ) {
		if (  buffer[ 1 ] == 0x02 ) {
			if (  buffer[ 2 ] == active_mode ) {
				asm( "nop" );
				return 1;
			}
		}
	}

	return 0;
}

void nR52Emit( uint8_t * buffer ) {
	uint64_t start_time = HAL_GetTick();



	char buf[ 35 ];
	char rx_buffer[ 35 ];
	char tx_buffer[ 35 ];
	buf[ 0 ] = READ_OP;
	buf[ 1 ] = 10;
	buf[ 2 ] = 0;
	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );
	delay_ns();

	do {
		HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 0 );
		HAL_IWDG_Refresh( &hiwdg );
		delay_ns();
		if ( HAL_SPI_TransmitReceive( &hspi1, buf, rx_buffer, 3, 1000 ) != HAL_OK ) {
			asm( "nop" );
			return 0;
		}

		HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );

		if ( ( start_time + 3000 ) < HAL_GetTick() )
			return 0;

	}while( rx_buffer[ 0 ] != 0x20 ||  rx_buffer[ 1 ] != 0x60 ||  rx_buffer[ 2 ] != 0x20  );


	for( uint8_t x = 0; x < 32; x++ ) {
		buf[ x+3 ] = buffer[ x ];

	}

	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 0 );
	delay_ns();
	if ( HAL_SPI_TransmitReceive( &hspi1, buf, rx_buffer, 35, 1000 ) != HAL_OK ) {
		asm( "nop" );
		HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );
		return 0;
	}
	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );

}


uint8_t nRF52WriteMultiRegister( uint8_t reg, uint8_t * data ) {
	uint8_t rx_buffer[ 3 ];
	uint8_t tx_buffer[ 3 ];

	tx_buffer[0] = READ_OP;
	tx_buffer[1] = reg;
	tx_buffer[2] = 0;

	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );
	delay_ns();
	uint64_t start_time = HAL_GetTick();

	do {
		HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 0 );
		delay_ns();
		if ( HAL_SPI_TransmitReceive( &hspi1, tx_buffer, rx_buffer, 3, 1000 ) != HAL_OK ) {
			asm( "nop" );
			return 0;
		}

		HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );

		if ( ( start_time + 3000 ) < HAL_GetTick() )
			return 0;

	}while( rx_buffer[ 0 ] != 0x20 ||  rx_buffer[ 1 ] != 0x60 ||  rx_buffer[ 2 ] != 0x20  );


	tx_buffer[0] = READ_OP;
	tx_buffer[1] = reg;
	tx_buffer[2] = 0;

	for( uint8_t x = 0; x < 32; x++ ) {
		tx_buffer[ x + 3 ] = data[ x ];
	}

	delay_ns();
	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 0 );
	delay_ns();
	if ( HAL_SPI_TransmitReceive( &hspi1, tx_buffer, rx_buffer, 3, 1000 ) != HAL_OK ) {
		asm( "nop" );
		HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );
		return 0;
	}
	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );
	return 1;
}

uint8_t nRF52WriteRegister( uint8_t reg, uint8_t data ) {
	uint8_t rx_buffer[ 3 ];
	uint8_t tx_buffer[ 3 ];

	tx_buffer[0] = READ_OP;
	tx_buffer[1] = data;
	tx_buffer[2] = 0;

	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );
	delay_ns();
	uint64_t start_time = HAL_GetTick();

	do {
		HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 0 );
		HAL_IWDG_Refresh( &hiwdg );
		delay_ns();
		if ( HAL_SPI_TransmitReceive( &hspi1, tx_buffer, rx_buffer, 3, 1000 ) != HAL_OK ) {
			asm( "nop" );
			return 0;
		}

		HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );
		delay_ns();
		if ( ( start_time + 3000 ) < HAL_GetTick() )
			return 0;

	}while( rx_buffer[ 0 ] != 0x20 ||  rx_buffer[ 1 ] != 0x60 ||  rx_buffer[ 2 ] != 0x20  );


	tx_buffer[0] = READ_OP;
	tx_buffer[1] = data;
	tx_buffer[2] = 0;
	delay_ns();
	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 0 );
	delay_ns();
	if ( HAL_SPI_TransmitReceive( &hspi1, tx_buffer, rx_buffer, 3, 1000 ) != HAL_OK ) {
		asm( "nop" );
		HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );
		return 0;
	}
	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );
	return 1;
}

void delay_ns() {
	for( uint64_t x = 0; x < 150; x++ ) {
		asm( "nop" );
	}
}

uint8_t nRF52ReadRcvRegister(  uint8_t * buffer ) {
	uint8_t rx_buffer[ 3 ];
	uint8_t tx_buffer[ 3 ];


	tx_buffer[0] = READ_OP;
	tx_buffer[1] = 2;
	tx_buffer[2] = 0;

	HAL_IWDG_Refresh( &hiwdg );
	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 0 );
	delay_ns();
	if ( HAL_SPI_TransmitReceive( &hspi1, tx_buffer, buffer, 32, 1000 ) != HAL_OK ) {
		asm( "nop" );
		return 0;
	}

	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );


	return 1;
}

uint16_t getLairdBootloaderPackets() {

	char buffer[50];
	nRF52ReadRegister( 9, buffer, 3 );


	return ( ( buffer[ 1 ] << 8 ) + buffer[ 2 ] );

}

uint8_t nRF52ReadRegister( uint8_t reg, uint8_t * buffer, uint8_t len ) {
	uint8_t rx_buffer[ 3 ];
	uint8_t tx_buffer[ 3 ];
	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );
	delay_ns();

	tx_buffer[0] = READ_OP;
	tx_buffer[1] = reg;
	tx_buffer[2] = 0;
	uint64_t start_time = HAL_GetTick();
	do {
		HAL_IWDG_Refresh( &hiwdg );
		HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 0 );
		delay_ns();
		if ( HAL_SPI_TransmitReceive( &hspi1, tx_buffer, rx_buffer, 3, 1000 ) != HAL_OK ) {
			asm( "nop" );
			return 0;
		}

		HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );
		delay_ns();
		if ( ( start_time + 3000 ) < HAL_GetTick() )
			return 0;
	}while( rx_buffer[ 0 ] != 0x20 ||  rx_buffer[ 1 ] != 0x60 ||  rx_buffer[ 2 ] != 0x20  );

	tx_buffer[0] = 0;
	tx_buffer[1] = 0;
	tx_buffer[2] = 0;

	delay_ns();
	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 0 );
	delay_ns();
	if ( HAL_SPI_TransmitReceive( &hspi1, tx_buffer, buffer, len, 1000 ) != HAL_OK ) {
		asm( "nop" );
		return 0;
	}


	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );

	delay_ns();
	return 1;
}


uint8_t nRF52deinit() {

	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 0 );
	HAL_GPIO_WritePin( SPI_EN_Port, SPI_EN_Pin, 0 );
	HAL_Delay( 100 );
	HAL_GPIO_WritePin( BOOST_EN_Port, BOOST_EN_Pin, 0 );
	HAL_Delay( 100 );
	HAL_SPI_DeInit( &hspi1 );

	return 1;
}

void nRF52StartReceiveMode( uint8_t mode ) {
	if ( mode == RECEIVE_BOOTLOADER )
		nRF52WriteRegister( 0, 5 );
	else if ( mode == RECEIVE_RESIDENTIAL_PARKING )
		nRF52WriteRegister( 0, 6 );
	else if ( mode == RECEIVE_COMMANDS )
		nRF52WriteRegister( 0, 7 );
	else if ( mode == RECEIVE_LAID_BOOTLOADER )
		nRF52WriteRegister( 0, 8 );

}

uint8_t nRF52init() {


	HAL_GPIO_WritePin( SPI_EN_Port, SPI_EN_Pin, 0 );
	HAL_Delay( 100 );
	HAL_GPIO_WritePin( BOOST_EN_Port, BOOST_EN_Pin, 0 );
	HAL_Delay( 100 );
	HAL_GPIO_WritePin( BOOST_EN_Port, BOOST_EN_Pin, 1 );
	HAL_GPIO_WritePin( SPI_EN_Port, SPI_EN_Pin, 1 );
	HAL_Delay( 1 );
	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );
	HAL_GPIO_WritePin( NRF_CSN_Port, NRF_CSN_Pin, 1 );

	HAL_Delay( 10 );

	MX_SPI1_Init();
	HAL_Delay( 10 );




	return 1;

}

uint8_t nRF52SetIRQPin() {
	 nRF52WriteRegister( 0, 11 );
#ifdef RADAR_VERSION
	/*
	char buffer[ 32 ];

	 nRF52WriteRegister( 0, 11 );
	 */
#endif

}

uint8_t nRF52UnsetIRQPin() {
	 nRF52WriteMultiRegister( 0, 12 );
#ifdef RADAR_VERSION
	/*
	char buffer[ 32 ];

	 nRF52WriteMultiRegister( 0, 12 );
	 */
#endif
}

uint8_t nRF52PacketReceived() {

	return HAL_GPIO_ReadPin( NRF_IRQ_Port, NRF_IRQ_Pin );

}

uint8_t nRF52getPacket( uint8_t * buffer ) {

	if ( nRF52ReadRegister( PACKET_CONT, buffer, 32 ) == 0 ) {
		return 0;
	}

	return 1;
}

