/*
 * X111.c
 *
 *  Created on: Jun 27, 2019
 *      Author: Tomas
 */


#include "X111.h"

void initRadar() {

	HAL_GPIO_WritePin( SPI_EN_Port, SPI_EN_Pin, 0 );
	HAL_Delay( 100 );
	HAL_GPIO_WritePin( BOOST_EN_Port, BOOST_EN_Pin, 0 );
	HAL_Delay( 100 );
	HAL_GPIO_WritePin( BOOST_EN_Port, BOOST_EN_Pin, 1 );
	HAL_Delay( 1 );
	HAL_GPIO_WritePin( NRF_CSN_Port, NRF_CSN_Pin, 1 );
	HAL_GPIO_WritePin( SPI_EN_Port, SPI_EN_Pin, 1 );
	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 1 );
	HAL_Delay( 10 );
	MX_SPI1_Init();

}

uint8_t isRadarPresent() {

	char buffer_resp[ 50 ];
	char buffer[ 2 ];
	buffer[0] = 0;
	buffer[1] = 0;


	for( uint8_t x = 0; x < 20; x++ ) {
		buffer_resp[ x ] = 0;
	}

	sendRequest( READ_OP, READ_WHO_AM_I, buffer, 2, &buffer_resp );

	if ( buffer_resp[ 0 ] == 0xA1 && buffer_resp[1] == 0xA3 ) {
		asm("nop");
		return 1;
	}
	return 0;
}


uint8_t setMinLength( uint16_t min, uint16_t length ) {
	char buffer[ 4 ];
	char buffer_resp[ 50 ];

	buffer[ 0 ] = ( min >> 8 ) & 0xFF;
	buffer[ 1 ] = ( min ) & 0xFF;
	buffer[ 2 ] = ( length >> 8 ) & 0xFF;
	buffer[ 3 ] = ( length ) & 0xFF;

	return sendRequest( WRITE_OP, WRITE_RADAR_DISTANCE_MIN_MAX, buffer, 4, buffer_resp );


}

void getRadarData( char * radar_data ) {
  //HAL_GPIO_WritePin( NRF_CSN_Port, NRF_CSN_Pin, 0 );

  initRadar();

  HAL_IWDG_Refresh( &hiwdg );
  HAL_Delay( 15000 );
  HAL_IWDG_Refresh( &hiwdg );

  if ( isRadarPresent() == 1 ) {
	  radarAssertIRQ();
	  HAL_IWDG_Refresh( &hiwdg );
	  setMinLength( RADAR_MIN_RANGE, RADAR_LENGTH_RANGE );
	  setAmplitudeRange( RADAR_MIN_AMPLITUDE );

	  setNewSweep( radar_data );

	  uint16_t sweep_results_size = ( radar_data[ 0 ] << 8 ) + radar_data[ 1 ];
  }



  radarDeinit();

}

void radarDeinit() {
	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, 0 );
	HAL_GPIO_WritePin( SPI_EN_Port, SPI_EN_Pin, 0 );
	HAL_Delay( 100 );
	HAL_GPIO_WritePin( BOOST_EN_Port, BOOST_EN_Pin, 0 );
	HAL_Delay( 100 );
	HAL_SPI_DeInit( &hspi1 );

}

uint8_t sendRequest( uint8_t type, uint8_t op_code, char * op_buffer, uint8_t op_buffer_size, char * response_buffer ) {

	char request_buffer[ 150 ];
	char request_buffer1[ 50 ];
	buildOp( type, op_code, request_buffer);

	uint64_t time = HAL_GetTick();

	uint8_t ok = 0;

	for( uint8_t x = 0; x < 3; x++ ) {
		request_buffer1[ x ] = 0;

	}

	while( (time + 3000) > HAL_GetTick())  {
		HAL_GPIO_WritePin( NRF_CSN_Port, NRF_CSN_Pin, 1 );
		HAL_Delay( 1 );
		HAL_GPIO_WritePin( NRF_CSN_Port, NRF_CSN_Pin, 0 );
		HAL_Delay( 1 );
		  HAL_IWDG_Refresh( &hiwdg );
		if ( HAL_SPI_TransmitReceive( &hspi1, request_buffer, request_buffer1,  3, 4000 ) == HAL_OK  ) {
			if ( request_buffer1[ 0 ] == 0x20 && request_buffer1[1] == 0x40 && request_buffer1[ 2 ] == 0x20 ) {
				ok = 1;
				break;
			}
		}

		HAL_Delay( 200 );
	}

	if ( ok == 0 )
		return 0;

	time = HAL_GetTick();
	HAL_Delay( 500 );
	ok = 0;
	while( (time + 3000) > HAL_GetTick())  {
		if ( HAL_SPI_TransmitReceive( &hspi1, op_buffer, response_buffer, op_buffer_size, 1000 ) == HAL_OK  ) {
			ok = 1;
			break;
		}
		  HAL_IWDG_Refresh( &hiwdg );
	}
	HAL_Delay( 200 );
	if ( ok == 0 )
		return 0;


	return 1;
}


void buildOp( uint8_t op_type, uint8_t op_code, char * buffer ) {

	buffer[ 0 ] = op_type;
	buffer[ 1 ] = op_code;
	buffer[ 2 ] = 0;


}

void radarFreeIRQ() {
	char buffer_resp[ 50 ];
	char buffer[ 2 ];
	buffer[0] = 0;
	buffer[1] = 0;


	sendRequest( READ_OP, FREE_IRQ_PIN, buffer, 2, &buffer_resp );

	if ( buffer_resp[ 0 ] == 0xA1 && buffer_resp[1] == 0xA3 )
		return 1;
}

uint8_t radarAssertIRQ() {
	char buffer_resp[ 50 ];
	char buffer[ 2 ];
	buffer[0] = 0;
	buffer[1] = 0;


	sendRequest( READ_OP, ASSERT_IRQ_PIN, buffer, 2, &buffer_resp );

	if ( buffer_resp[ 0 ] == 0xA1 && buffer_resp[1] == 0xA5 )
		return 1;

	return 0;
}

uint8_t setNewSweep( char * sweep_results ) {
	char buffer[ 4 ];
	char buffer_resp[ 50 ];

	buffer[ 0 ] = 1;

	sendRequest( WRITE_OP, WRITE_RADAR_SWEEP, buffer, 1, buffer_resp );

	uint64_t start_time = HAL_GetTick();
	uint8_t ok = 0;
	while( (start_time + 65000) > HAL_GetTick() ) {
		if ( HAL_GPIO_ReadPin( NRF_IRQ_Port, NRF_IRQ_Pin ) == 1 ) {
			ok = 1;
			break;
		}
		HAL_IWDG_Refresh( &hiwdg );
	}
	HAL_Delay( 9000 );

	if ( ok == 1 ) {
		getSweepResults( sweep_results );
		return 1;
	}

	return 0;
}

uint8_t isSweepComplete() {

	if ( HAL_GPIO_ReadPin( NRF_IRQ_Port, NRF_IRQ_Pin ) == 1 ) {
		return 1;
	}

	return 0;
}

uint8_t getSweepResults( char * buffer ) {
	char buffer_resp[ 102 ];

	return sendRequest( READ_OP, READ_RESULT, buffer_resp, 102, buffer );

}

uint8_t setAmplitudeRange( uint16_t amp ){
	char buffer[ 4 ];
	char buffer_resp[ 50 ];

	buffer[ 0 ] = ( amp >> 8 ) & 0xFF;
	buffer[ 1 ] = ( amp ) & 0xFF;



	return sendRequest( WRITE_OP, WRITE_MIN_AMPLITUDE_FOR_DETECTION, buffer, 2, buffer_resp );
}
