/*
 * battery.c
 *
 *  Created on: May 2, 2017
 *      Author: fiala
 */

#include "battery.h"


#define VREFINT_CAL ((uint16_t*) ((uint32_t) 0x1FF80078))

extern ADC_HandleTypeDef hadc;
extern struct MM001 radio;
extern UART_HandleTypeDef huart_radio;

struct batt_t batt;

/* ADC init function */
static void MX_ADC_Init(void)
{

  ADC_ChannelConfTypeDef sConfig;

    /**Configure the global features of the ADC (Clock, Resolution, Data Alignment and number of conversion)
    */
  hadc.Instance = ADC1;
  hadc.Init.OversamplingMode = DISABLE;
  hadc.Init.ClockPrescaler = ADC_CLOCK_SYNC_PCLK_DIV1;
  hadc.Init.Resolution = ADC_RESOLUTION_12B;
  hadc.Init.SamplingTime = ADC_SAMPLETIME_1CYCLE_5;
  hadc.Init.ScanConvMode = ADC_SCAN_DIRECTION_FORWARD;
  hadc.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc.Init.ContinuousConvMode = DISABLE;
  hadc.Init.DiscontinuousConvMode = ENABLE;
  hadc.Init.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_NONE;
  hadc.Init.ExternalTrigConv = ADC_SOFTWARE_START;
  hadc.Init.DMAContinuousRequests = DISABLE;
  hadc.Init.EOCSelection = ADC_EOC_SINGLE_CONV;
  hadc.Init.Overrun = ADC_OVR_DATA_PRESERVED;
  hadc.Init.LowPowerAutoWait = DISABLE;
  hadc.Init.LowPowerFrequencyMode = DISABLE;
  hadc.Init.LowPowerAutoPowerOff = ENABLE;
  if (HAL_ADC_Init(&hadc) != HAL_OK)
  {  }

    /**Configure for the selected ADC regular channel to be converted.
    */
  sConfig.Channel = ADC_CHANNEL_0;
  sConfig.Rank = ADC_RANK_CHANNEL_NUMBER;
  if (HAL_ADC_ConfigChannel(&hadc, &sConfig) != HAL_OK)
  {  }

    /**Configure for the selected ADC regular channel to be converted.
    */

  sConfig.Channel = ADC_CHANNEL_VREFINT;
  sConfig.Rank = ADC_RANK_CHANNEL_NUMBER;
  if (HAL_ADC_ConfigChannel(&hadc, &sConfig) != HAL_OK)
  {  }

}


void batt_init(void)
{
	batt.calib_value = 100;
}

void setBatReadPin( uint8_t state ) {
	HAL_GPIO_WritePin( BAT_EN_GPIO_Port, BAT_EN_Pin, state );
}

uint32_t readADC(void)
{
	uint32_t adcval = 0;
	uint64_t start_time = HAL_GetTick();

	HAL_ADCEx_Calibration_Start(&hadc, ADC_SINGLE_ENDED);

	uint32_t f = HAL_ADC_GetValue( &hadc );



	while(HAL_ADC_Start(&hadc) != HAL_OK ) {
		//HAL_UART_Transmit( &huart_radio, "WAAAAAAAAA\n", strlen( "WAAAAAAAAA\n" ), 100);
		if (  ( start_time + 2000 ) < HAL_GetTick() ) {
			break;
		}
		continue;
	}

	if ( f == 1 )
		asm( "nop" );

	if (HAL_ADC_PollForConversion(&hadc, 1000) == HAL_OK)
	{
		while( (HAL_ADC_GetState(&hadc) & HAL_ADC_STATE_REG_EOC) != HAL_ADC_STATE_REG_EOC) {
			continue;
		}
		adcval = HAL_ADC_GetValue( &hadc );
	}

	HAL_ADC_Stop(&hadc);
	return adcval;

}
float batt_adc_to_voltage(uint32_t adcBAT, uint32_t adcCPU)
{
	float vbat = 3.0 * (float)( *VREFINT_CAL) * (float)adcBAT * 3.0 / ( (float) adcCPU );
	return(vbat);
}

uint32_t batt_get_voltage(void)
{
	uint32_t adcBAT = 0;
	uint32_t adcCPU = 0;

	HAL_GPIO_WritePin( BOOST_EN_Port, BOOST_EN_Pin, 1 );
	setBatReadPin( 1 );

	HAL_Delay( 100 );
	MX_ADC_Init();

	HAL_ADCEx_Calibration_Start( &hadc, ADC_SINGLE_ENDED );

	uint32_t d = HAL_ADC_GetValue( &hadc );

	if ( d == 30 )
		asm( "nop" );

	adcBAT = readADC();
	adcCPU = readADC();


	HAL_ADC_DeInit( &hadc );
	setBatReadPin( 0 );
	float vbat = ((adcBAT*2)*0.732421875);
	float vbat2 = ((adcCPU)*0.732421875);

	if ( vbat2 == 10 )
		asm( "nop" );

	uint32_t vbat1 = lrint( vbat );

	HAL_GPIO_WritePin( BOOST_EN_Port, BOOST_EN_Pin, 0 );

	return vbat1;
}

uint8_t batt_state(float vbat)
{


	//float vbat = 3.2;
	uint8_t batt_state = (uint8_t)round( 7 * (vbat - BATT_MIN) / (BATT_MAX - BATT_MIN) );

	if (batt_state > 7)
		batt_state = 0;

	//uint8_t data[2];
	//get_dec_str(data, 2, batt_state );
	//MM001_sendDataDontWait( &radio, data, 2 );


	//end
	return batt_state;
}

uint8_t getBattEncoded() {
	float ff = batt_measure();
	//ff = ff * 2;
	float val = (  ( ff - BATT_MIN ) / (BATT_MAX - BATT_MIN) ) * 100;

	uint8_t batt_measure = rint( val /6.7 );
	return batt_measure;

}

float batt_measure( void )
{
	MX_ADC_Init();
	while(HAL_ADCEx_Calibration_Start(&hadc, ADC_SINGLE_ENDED) != HAL_OK)
		{continue;}

	HAL_GPIO_WritePin(BAT_EN_GPIO_Port, BAT_EN_Pin, GPIO_PIN_SET);
	HAL_Delay(200);

	float voltage = batt_get_voltage();

	//HAL_Delay(200);

	HAL_GPIO_WritePin(BAT_EN_GPIO_Port, BAT_EN_Pin, GPIO_PIN_RESET);

	if( HAL_OK != HAL_ADC_Stop(&hadc))
		HAL_UART_Transmit( &huart2, "HAL_NOT_OK0\n" , strlen("HAL_NOT_OK1\n"), 1000);

	HAL_Delay(200);

	ADC_ChannelConfTypeDef sConfig;
	sConfig.Channel = ADC_CHANNEL_VREFINT;
	sConfig.Rank = ADC_RANK_NONE;
	if( HAL_OK != HAL_ADC_ConfigChannel(&hadc, &sConfig))
		HAL_UART_Transmit( &huart2, "HAL_NOT_OK1\n" , strlen("HAL_NOT_OK1\n"), 1000);

	sConfig.Channel = ADC_CHANNEL_1;
	sConfig.Rank = ADC_RANK_NONE;

	if( HAL_OK != HAL_ADC_ConfigChannel(&hadc, &sConfig))
		HAL_UART_Transmit( &huart2, "HAL_NOT_OK2\n" , strlen("HAL_NOT_OK1\n"), 1000);

	if( HAL_OK != HAL_ADC_DeInit(&hadc))
		HAL_UART_Transmit( &huart2, "HAL_NOT_OK3\n" , strlen("HAL_NOT_OK1\n"), 1000);

	return voltage;
}

void batt_measure_periodic_IT_start( void )
{

	ADC_ChannelConfTypeDef sConfig;

	/**Configure the global features of the ADC (Clock, Resolution, Data Alignment and number of conversion)
	*/
	hadc.Instance = ADC1;
	hadc.Init.OversamplingMode = DISABLE;
	hadc.Init.ClockPrescaler = ADC_CLOCK_SYNC_PCLK_DIV1;
	//hadc.Init.ClockPrescaler = ADC_CLOCK_ASYNC_DIV256;
	hadc.Init.Resolution = ADC_RESOLUTION_12B;
	hadc.Init.SamplingTime = ADC_SAMPLETIME_160CYCLES_5;
	hadc.Init.ScanConvMode = ADC_SCAN_DIRECTION_FORWARD;
	hadc.Init.DataAlign = ADC_DATAALIGN_RIGHT;
	hadc.Init.ContinuousConvMode = ENABLE;
	hadc.Init.DiscontinuousConvMode = DISABLE;
	hadc.Init.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_NONE;
	hadc.Init.ExternalTrigConv = ADC_SOFTWARE_START;
	hadc.Init.DMAContinuousRequests = DISABLE;
	hadc.Init.EOCSelection = ADC_EOC_SINGLE_CONV;
	hadc.Init.Overrun = ADC_OVR_DATA_PRESERVED;
	hadc.Init.LowPowerAutoWait = DISABLE;
	hadc.Init.LowPowerFrequencyMode = ENABLE;
	hadc.Init.LowPowerAutoPowerOff = ENABLE;
	if (HAL_ADC_Init(&hadc) != HAL_OK)
	{  }

	/**Configure for the selected ADC regular channel to be converted.
	*/
	sConfig.Channel = ADC_CHANNEL_4;
	sConfig.Rank = ADC_RANK_CHANNEL_NUMBER;
	if (HAL_ADC_ConfigChannel(&hadc, &sConfig) != HAL_OK)
	{  }

	/**Configure for the selected ADC regular channel to be converted.
	*/
	sConfig.Channel = ADC_CHANNEL_VREFINT;
	if (HAL_ADC_ConfigChannel(&hadc, &sConfig) != HAL_OK)
	{  }

	HAL_NVIC_SetPriority(ADC1_COMP_IRQn, 2, 0);
	HAL_NVIC_EnableIRQ(ADC1_COMP_IRQn);

	while(HAL_ADCEx_Calibration_Start(&hadc, ADC_SINGLE_ENDED) != HAL_OK)
		{continue;}

	HAL_GPIO_WritePin(BAT_EN_GPIO_Port, BAT_EN_Pin, GPIO_PIN_SET);
	HAL_Delay(100);

	uint8_t adc_it_cnt = 0;
	HAL_ADC_Start_IT(&hadc);

}
void batt_measure_periodic_IT_stop(void)
{


	HAL_GPIO_WritePin(BAT_EN_GPIO_Port, BAT_EN_Pin, GPIO_PIN_RESET);

	if( HAL_OK != HAL_ADC_Stop_IT(&hadc))
	{}

	ADC_ChannelConfTypeDef sConfig;
	sConfig.Channel = ADC_CHANNEL_VREFINT;
	sConfig.Rank = ADC_RANK_NONE;
	if( HAL_OK != HAL_ADC_ConfigChannel(&hadc, &sConfig))
	{}
	sConfig.Channel = ADC_CHANNEL_1;
	sConfig.Rank = ADC_RANK_NONE;
	if( HAL_OK != HAL_ADC_ConfigChannel(&hadc, &sConfig))
	{}
	if( HAL_OK != HAL_ADC_DeInit(&hadc))
	{}

}
