/*
 * bootloader.c
 *
 *  Created on: Nov 2, 2018
 *      Author: Tomas
 */

#include "bootloader.h"



#define CRC16 0x8005



void SystemClock_Config1(void)
{

  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_PeriphCLKInitTypeDef PeriphClkInit;

    /**Configure the main internal regulator output voltage
    */
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

    /**Initializes the CPU, AHB and APB busses clocks
    */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI|RCC_OSCILLATORTYPE_LSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = 16;
  RCC_OscInitStruct.LSIState = RCC_LSI_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLLMUL_4;
  RCC_OscInitStruct.PLL.PLLDIV = RCC_PLLDIV_2;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
  }

    /**Initializes the CPU, AHB and APB busses clocks
    */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_1) != HAL_OK)
  {
  }

  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_USART1|RCC_PERIPHCLK_USART2
                              |RCC_PERIPHCLK_I2C1|RCC_PERIPHCLK_RTC;
  PeriphClkInit.Usart1ClockSelection = RCC_USART1CLKSOURCE_SYSCLK;
  PeriphClkInit.Usart2ClockSelection = RCC_USART2CLKSOURCE_SYSCLK;
  PeriphClkInit.I2c1ClockSelection = RCC_I2C1CLKSOURCE_SYSCLK;
  PeriphClkInit.RTCClockSelection = RCC_RTCCLKSOURCE_LSI;

  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

    /**Configure the Systick interrupt time
    */
  HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq()/1000);

    /**Configure the Systick
    */
  HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

  /* SysTick_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(SysTick_IRQn, 0, 0);
}


unsigned int doCrc( char *data, int size)
{
  int i, j;
  unsigned int crc = 0;
  for (i = 0; i < size; i++) {
    unsigned int xr = data[i] << 8;
    crc = crc ^ xr;

    for (j = 0; j < 8; j++)
    {
      if (crc & 0x8000) {
        crc = (crc << 1);
        crc = crc ^ 0x1021;
      }
      else {
        crc = crc << 1;
      }
    }
  }
  crc = crc & 0xFFFF;
  return crc;
}

void setRcvPacket( uint64_t packet_num ) {
	rcv_packet[ (packet_num/8) ] ^= (1  << (packet_num%8));
}

uint8_t isPacketUsed( uint64_t packet_num ) {
	return rcv_packet[ (packet_num/8) ] & (1 << (packet_num%8));
}

int bootloaderInit() {
	size = 0;
	addrses = 0;
	number_of_distinct_packets = 0;
	time_last_update_progress = 0;
	file_crc = 0;
	total_received_size = 0;
	bootloader_last_sanity_check = HAL_GetTick();
	HAL_GPIO_WritePin(BOOST_EN_Port, BOOST_EN_Pin, GPIO_PIN_SET);
    HAL_Delay( 500 );
	HAL_GPIO_WritePin(SPI_EN_Port, SPI_EN_Pin, GPIO_PIN_SET);
	HAL_IWDG_Refresh( &hiwdg );
	flashActive();
	HAL_IWDG_Refresh( &hiwdg );
	HAL_GPIO_WritePin( NRF_CSN_Port, NRF_CSN_Pin, GPIO_PIN_RESET );
	HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, GPIO_PIN_RESET );
	SystemClock_Config1();
	HAL_IWDG_Refresh( &hiwdg );
	flashDisable();
	HAL_IWDG_Refresh( &hiwdg );
    HAL_Delay( 500 );
    MX_SPI1_Init();
    HAL_Delay( 100 );
    HAL_IWDG_Refresh( &hiwdg );
    flashChipErase();
    HAL_IWDG_Refresh( &hiwdg );
    RTC->BKP4R = 0;

    initNRF();
    startNRF( RECEIVE_BOOTLOADER );
    nRF52SetIRQPin();
    HAL_IWDG_Refresh( &hiwdg );


	for( uint16_t x = 0; x < 500; x++ ) {
		  rcv_packet[ x ] = 0;
	}

	waiting_for_packet = -1;

	return 1;
}


void bootloaderStep() {
	uint8_t buffer1[ 32 ];
	HAL_IWDG_Refresh( &hiwdg );

	if ( (bootloader_last_sanity_check + 60000) < HAL_GetTick() ) {
		if ( nrfSanityCheck( 5 ) == 0 ) {

				HAL_SPI_DeInit( &hspi1 );
				HAL_GPIO_WritePin(BOOST_EN_Port, BOOST_EN_Pin, GPIO_PIN_RESET);
				HAL_GPIO_WritePin(SPI_EN_Port, SPI_EN_Pin, GPIO_PIN_RESET);
				flashActive();
				HAL_GPIO_WritePin( NRF_CSN_Port, NRF_CSN_Pin, GPIO_PIN_RESET );
				HAL_GPIO_WritePin( NRF_CE_Port, NRF_CE_Pin, GPIO_PIN_RESET );
				HAL_Delay( 500 );
				initNRF();
				startNRF( RECEIVE_BOOTLOADER );

		}else{
			bootloader_last_sanity_check = HAL_GetTick();
		}

	}


		while( nrfIsPacketWaiting() ) {


				HAL_IWDG_Refresh( &hiwdg );

				nrfGetPacket( &buffer1 );



				HAL_IWDG_Refresh( &hiwdg );

				addrses = ( buffer1[ 0 ] << 8 );
				addrses += ( buffer1[ 1 ] );
				uint16_t crc = doCrc( (char *)buffer1, 30 );

				uint16_t rcv_crc = ((buffer1[30] << 8 ) + buffer1[ 31 ]);
				if ( crc != rcv_crc ) {
					asm( "nop" );
					continue;
				}


				if ( addrses == 0xFFFF && size == 0) {
					size = ( buffer1[ 2 ] << 8 ) + buffer1[ 3 ];

					uint8_t sizes[2];
					sizes[ 0 ] = buffer1[ 2 ];
					sizes[ 1 ] = buffer1[ 3 ];

					file_crc = ( buffer1[ 4 ] << 8 ) + buffer1[ 5 ];
					uint8_t size_check[2];
					size_check[ 0 ] = 0;
					size_check[ 1 ] = 0;


					uint8_t id_check = 1;




					for( uint8_t x = 0; x < ID_SIZE; x++ ) {
						if ( buffer1[ x+6 ] != un_device_id[ x ] ) {
							id_check = 0;
						}
					}

					if ( id_check == 1 && buffer1[ 14 ] == 1 ) {
						stopNRFMode();
						nrf_mode = NO_NRF_MODE;
						nrf_startup = 0;
						return;
					}

					if ( id_check == 1 && buffer1[ 14 ] == 3 ) {
						RTC->BKP4R = 0x22;
					}



					uint32_t start_time = HAL_GetTick();
					uint8_t error = 0;

					while(1) {
						if ( (start_time + 3000) < HAL_GetTick() ) {
							error = 1;
							break;
						}
						flashWriteData( sizes ,2, 0xFA01 );
						HAL_IWDG_Refresh( &hiwdg );

						flashReadData( size_check, 2, 0xFA01 );
						if ( size_check[ 0 ] == sizes[ 0 ] && size_check[ 1 ] == sizes[ 1 ]  )
							break;
					}

					if ( error == 1 ) {
						nrf_startup = 99;
						send_msg_err( 0,0,0,0,1,0,1, 0 );
						return;
					}

					if ( buffer1[14] == 2 ) {
						start_time = HAL_GetTick();
						error = 0;

						while(1) {
							uint8_t hash[ 10 ];
							uint8_t hash_check[ 10 ];

							if ( (start_time + 3000) < HAL_GetTick() ) {
								error = 1;
								break;
							}

							for( uint8_t x = 0; x < 10; x++ ) {
								hash[ x ] = buffer1[ 15 + x ];
							}

							flashWriteData( hash, 10, 0x1CCF0 );

							for( uint8_t x = 0; x < 10; x++ ) {
								hash_check[ x ] = 0;
							}

							flashReadData( hash_check, 10, 0x1CCF0 );
							uint8_t check = 1;
							for( uint8_t x = 0; x < 10; x++ ) {
								if ( hash_check[ x ] != hash[ x ] ) {
									check = 0;
								}
							}
							if ( check == 1 ) {
								break;
							}
						}

						if ( error == 1 ) {
							nrf_startup = 99;
							send_msg_err( 0,0,0,0,1,0,2, 0 );
							return;
						}


					}


					continue;
				}else if( addrses == 0xFFFF && size != 0 ) {
					continue;
				}


				if ( (addrses*28) >= 54000 ) {
					continue;
				}


				if ( isPacketUsed( addrses ) != 0 ) {
					continue;
				}else{
					setRcvPacket( addrses );
					number_of_distinct_packets++;

					total_received_size += 28;
				}

				if (  number_of_distinct_packets % 100 == 0 ) {
					blinkLed( 5 );

				}

				for( int x = 0; x < 28; x++ ) {
					buffer1[ x ] = buffer1[ x + 2 ];
				}
				uint8_t datas[ 28 ];

				for( int x = 0; x < 28; x++ ) {
					datas[ x ] = 0;
				}


				uint8_t error = 1;

				uint32_t write_address = addrses*28 ;
				uint8_t number_of_failures = 0;
				while( error != 0 ) {
					for( int x = 0; x < 28; x++ ) {
						datas[ x ] = 0;
					}

					error = 0;

					for( int x = 0; x < 28; x++ ) {
						flashWriteData( buffer1+x, 1, write_address+x );
					}

					flashReadData( datas, 28, write_address );
					for( int x = 0;x < 28; x++ ) {
						if ( buffer1[ x ] != datas[ x ] ) {
							//flashOff();
							number_of_failures++;
							if ( number_of_failures > 100 ) {
								nrf_mode = NO_NRF_MODE;
								send_msg_err( 0,0,0,0,1,0,3, 0 );
								bootloaderEnd();
								nrf_startup = 99;
								return;
							}
							error++;
							//HAL_Delay( 2000 );
							//flashOn();
							break;
						}
					}
				}



				if ( total_received_size >= size && size != 0 ) {
					HAL_UART_Transmit( &huart2, "Switching to bootloader\n", strlen( "Switching to bootloader\n" ), 1000 );
					RTC->BKP0R = 122;
					uint16_t calculated_file_crc = 0;
					nrf_mode = NO_NRF_MODE;
					send_msg_bootloader( 1,0,0,0 );
					/*
					for( int x = 0; x < size; x+=1 ) {
						flashReadData( datas, 1, x );
						calculated_file_crc = gen_crc16( datas, 1, calculated_file_crc );
					}
	*/
					if ( file_crc != calculated_file_crc ) {
						asm( "nop" );
					}
					nvmWriteByte( BASELINE_VAL, data_x.bellum );
					nvmWriteByte( BASELINE_VAL + 4, data_mag.bellum );
					nvmWriteByte( BASELINE_VAL + 8, data_rotation.bellum );
					nvmWriteByte( BASELINE_VAL + 12, data_total.bellum );
					nvmWriteByte( BASELINE_VAL + 16, data_mag_total.bellum );

					if ( pht_ons_data == 0 )
						nvmWriteByte( BASELINE_VAL+20, 2);
					else
						nvmWriteByte( BASELINE_VAL+20, 99);

					if ( RTC->BKP4R == 0x22 ) {
						RTC->BKP4R = 0;
						nvmWriteByte( BASELINE_VAL+20, 99);
					}


					//nvmWriteByte( BASELINE_VAL, data_x.bellum );

					send_msg_bootloader( 0,0,0,1 );

					  while( HAL_FLASH_Unlock() != HAL_OK ) {
						  continue;
					  }

					  while( HAL_FLASH_OB_Unlock() != HAL_OK ) {
						  continue;
					  }

					  FLASH_OBProgramInitTypeDef pOBInit;

					  HAL_FLASHEx_OBGetConfig( &pOBInit );

					  pOBInit.WRPState = OB_WRPSTATE_DISABLE;

					  while( HAL_FLASHEx_OBProgram( &pOBInit ) != HAL_OK ) {
						  continue;
					  }



					  while( HAL_FLASH_OB_Lock() != HAL_OK ) {
						  continue;
					  }

					  while( HAL_FLASH_Lock() != HAL_OK ) {
						  continue;
					  }

					  HAL_FLASH_OB_Launch();

					  NVIC_SystemReset();


					HAL_NVIC_SystemReset();
					break;

				}

		}

}

void bootloaderEnd() {
	nrf_mode = NO_NRF_MODE;
	stopNRFMode();

}
