/*
 * magnetometer.c
 *
 *  Created on: Aug 2, 2018
 *      Author: Tomas
 */
#include "magnetometer.h"

void softReset() {
	MX_I2C1_Init();
	int32_t total_x[10];
	int32_t total_y[10];
	int32_t total_z[10];

	for( int x = 0; x < 5; x++ ) {
		uint8_t res = MMC34160PJCalculateOffset();

		if ( res == 1 ) {
			send_msg_err( 1,0,0,0 );
			return;
		}

		total_x[x] = MMC34160PJ_offset.x;
		total_y[x] = MMC34160PJ_offset.y;
		total_z[x] = MMC34160PJ_offset.z;
	}

	uint8_t number_of_exchanges = 1;

	CircleBuffer sort_buffer;


	while( number_of_exchanges >= 1 ) {
		number_of_exchanges = 0;
		for ( int x = 0; x < 4; x++ ) {
			if ( total_x[ x ] > total_x[ x + 1 ] ) {
				number_of_exchanges++;
				int32_t temp = total_x[ x ];
				total_x[ x ] = total_x[ x + 1 ];
				total_x[ x + 1 ] = temp;
			}
		}
	}

	MMC34160PJ_offset.x = total_x[2] / 1;
	MMC34160PJ_offset.y = total_y[2] / 1;
	MMC34160PJ_offset.z = total_z[2] / 1;

	MMC34160PJuploadSettings();
	//setNewMeas();
	#ifdef DEBUG
			//send_msg_baseline_offset( data_x.bellum, MMC34160PJ_offset.x, MMC34160PJ_offset.y, MMC34160PJ_offset.z );
	#endif

	//HAL_I2C_DeInit( &hi2c1 );
}


void initAMD() {
	c_count = 0;
	current_MDR = 2;
}

void stepAMD( uint8_t c ) {
	uint8_t new_current_MDR = current_MDR;
	if ( current_MDR < 4 && c == 1 ) {
		new_current_MDR = 4;
		c_count = 0;
	}else if ( current_MDR == 4 && c == 0 && c_count >= 1 ) {
		new_current_MDR = 3;
		c_count = 0;
	}else if ( current_MDR == 3 && c == 0 && c_count >= 100 ) {
		new_current_MDR = 2;
		c_count = 0;
	}else if ( current_MDR == 2 && c == 0 && c_count >= 100 ) {
		new_current_MDR = 1;
		c_count = 0;
	}

	if ( c == 0 ) {
		c_count++;
	}

	if ( new_current_MDR != current_MDR ) {
		current_MDR =  new_current_MDR;
		setRTCFreq( current_MDR );
	}


}

void resetMagnetometer() {


	HAL_GPIO_WritePin( MAG_VDDIO_GPIO_Port, MAG_VDDIO_Pin, GPIO_PIN_RESET );
	HAL_Delay( 1500 );
	HAL_GPIO_WritePin( MAG_VDDIO_GPIO_Port, MAG_VDDIO_Pin, GPIO_PIN_SET );
	MX_I2C1_Init();

	HAL_Delay( 500 );

	MX_I2C1_Init();

	#ifdef LIS3MDL
		lis3mdl_basic_config();
	#endif
	#ifdef MMC34160PJ
		MMC34160PJCalculateOffset();
		MMC34160PJuploadSettings();
	#endif
	#ifdef BOTH_MAGNETOMETERS
		MMC34160PJuploadSettings();
		softReset();
		setNewMeas();
	#endif
	    HAL_Delay( 500 );
	   // HAL_I2C_DeInit( &hi2c1 );
#ifdef DEBUG
	#ifndef LIS3MDL
	  //send_multibyte_msg( "e1e1", 4 );
    #endif
#endif
}

void setHZData( uint8_t req_settings ) {
/*
	#ifdef HZ1
		#define LIS3MDL_FREQ 0b001
		#define MMC3416xPJ_FREQ 0b00

	#endif
	#ifdef HZ_2_5
		#define LIS3MDL_FREQ 0b010
		#define MMC3416xPJ_FREQ 0b00
	#endif
	#ifdef HZ_5
		#define LIS3MDL_FREQ 0b011
		#define MMC3416xPJ_FREQ 0b01
	#endif
	#ifdef HZ_10
		#define LIS3MDL_FREQ 0b100
		#define MMC3416xPJ_FREQ 0b01
	#endif
	#ifdef HZ_20
		#define LIS3MDL_FREQ 0b101
		#define MMC3416xPJ_FREQ 0b10
	#endif
	#ifdef HZ_50
		#define LIS3MDL_FREQ 0b110
		#define MMC3416xPJ_FREQ 0b11
	#endif
*/

}

uint8_t getData( Mag3110 * gg ) {

	#ifdef LIS3MDL
		getCurrentVal( gg );
		return 1;
	#endif
	#ifdef MMC34160PJ
		MMC34160PJgetData( gg );
	#endif
	#ifdef BOTH_MAGNETOMETERS
		uint8_t res = 0;

		uint8_t s = isMeasDone();
		if ( s == 1 ) {
			res = MMC34160PJgetData( gg );
		}

		setNewMeas();
		Mag3110 gg1;
		//getCurrentVal( &gg1 );
		return res;
	#endif
}
