/*
 * init_operational.c
 *
 *  Created on: Aug 1, 2017
 *      Author: fiala
 */

#include "mode_operational.h"
#include "nvm.h"


extern MagneticQueue mag_queue;


void magneticDelay( __IO uint32_t Delay )
{
  uint32_t tickstart = 0U;
  tickstart = HAL_GetTick();
  while((HAL_GetTick() - tickstart) < Delay)
  {
	    HAL_IWDG_Refresh( &hiwdg );

		if ( nrf_mode == NRF_MODE ) {
			bootloaderStep();
		}

  }
  HAL_IWDG_Refresh( &hiwdg );



}

void lowPowerMode(void)	//deinit peripetials, enter sleep mode, after sleep init and return
{

	HAL_IWDG_Refresh( &hiwdg );
	HAL_PWR_EnterSTOPMode(PWR_LOWPOWERREGULATOR_ON, PWR_SLEEPENTRY_WFI);

}

void mode_operational_init() {

	old_state = 0;
    mag_int = INTERRUPT;

    parking_init( &data_x );
    parking_init( &data_mag );
    parking_init( &data_rotation );
    parking_init( &data_total );
    parking_init( &data_mag_total );


	main_loop.park_state = 0;
	main_loop.total_int_count = 0;
	main_loop.mag_baseline_report_counter = 0;
	main_loop.mag_fcnt_count = 0;
	main_loop.temp_cnt = 0;
	main_loop.hard_mag_baseline_report_counter = 0;

	main_loop.last_msg_timeout = 0;
	MX_I2C1_Init();

	setNewMeas();
	setRTCFreq( 2 );
	initAMD();

	blinkLed( 1000 );

	data_rotation.threshold_up = 350;
	data_rotation.threshold_down = 300;
	data_rotation.threshold_intg = 100;

	data_mag.threshold_up = 120;
	data_mag.threshold_down = 90;
	data_mag.threshold_intg = 45;

	data_total.threshold_up = 125;
	data_total.threshold_down = 95;
	data_total.threshold_intg = 45;


	data_mag_total.threshold_up = 130;
	data_mag_total.threshold_down = 110;
	data_mag_total.threshold_intg = 30;


}


void mode_operational_loop_presentation( uint8_t type_init )
{
    mag_int = INTERRUPT;

	MX_I2C1_Init();

	setNewMeas();
	setRTCFreq( 2 );
	initAMD();

	blinkLed( 3000 );

	uint8_t parked = 0;
	while (1)
	{

			while( isQueueEmpty( &mag_queue ) == 0 ) {
				RTC->BKP1R = STATE_MAG_ACTIVE;
				Mag3110 val = getValue( &mag_queue );


				int64_t mag = sqrt( val.x*val.x + val.y*val.y+val.z*val.z);


				if ( mag > 4000 ) {

					if ( parked == 0 ) {

						  uint8_t resp = nRF52init();
						  startNRF( RECEIVE_RESIDENTIAL_PARKING );
						  if ( resp  == 0 ) {
							  asm( "nop" );
						  }

						  char buffer[ 50 ];
						  asm( "nop" );

						  uint64_t start_time = HAL_GetTick();
						  uint8_t found = 0;
						  uint8_t counts = 0;
						  uint32_t deveui_ids[9];
						  int16_t deveui_snr[9];

						  while( (start_time + 15000) > HAL_GetTick() ) {

							  if ( nRF52PacketReceived() == 1 ) {

								uint8_t buffer[32];

								if ( nRF52getPacket( &buffer ) == 1 ) {
									  if ( buffer[ 0 ] == 0xAA && buffer[ 1 ] == 0xAB ) {
										  //nRF52deinit();
										  found = 1;


										  deveui_ids[ counts ] = (buffer[ 3 ] << 24) + (buffer[ 2 ] << 16) + (buffer[ 1 ] << 8) + (buffer[ 0 ]);
										  deveui_snr[ counts ] = buffer[ 4 ];

										  counts = counts + 1;
										  break;
										  if ( counts > 10 )
											  break;
									  }
								}

							  }

						  }

						  if ( found == 1 ) {
							  HAL_GPIO_WritePin( LED_GPIO_Port, LED_Pin, 0 );
							  //send_msg_park_residential(main_loop.park_state_x, main_loop.park_state_y, main_loop.park_state_z, main_loop.park_state_x, deveui_ids, counts, POWER_20 );

						  }

						  if ( found == 0 ) {
							  blinkLed( 500 );
							  blinkLed( 500 );

							  //send_msg_park(0, 0,0, 0, 1, POWER_20 );
						  }

						  nRF52deinit();

					}

					parked = 1;

				}else{
					if ( parked == 1 ){
						blinkLed( 500 );
						//send_msg_park(0, 0,0, 0, 0, POWER_20 );
					}

					parked = 0;
					HAL_GPIO_WritePin( LED_GPIO_Port, LED_Pin, 1 );
				}

				#ifdef SD_LOGGER
					char data[ 500 ];
					sd_write_to_card( val.x, val.y, val.z, (uint8_t)data_x.state, data_x.b_old, data_x.a, data_x.diff );
				#endif


			}
			magneticStep();
	}
}

void magneticStep() {
	MX_I2C1_Init();
	HAL_IWDG_Refresh( &hiwdg );
	main_loop.idle_cnt++;




#ifdef SIGFOX
	if ( main_loop.rcv_counter <= 4000000000 ) {
		main_loop.rcv_counter++;
	}
#endif

	if ( nrf_mode == NRF_MODE ) {
		if ( time_last_update_progress < 64000000 ) {
			time_last_update_progress++;
		}
	}

	if ( main_loop.mag_reset_offset_count < 64000000 ) {
		main_loop.mag_reset_offset_count++;
	}
	if ( main_loop.last_msg_timeout < 2000000 ) {
		main_loop.last_msg_timeout++;
	}

	if ( main_loop.hard_mag_baseline_report_counter < 640000000 ) {
		main_loop.hard_mag_baseline_report_counter++;
	}

	if ( main_loop.command_mode_rcvr < 640000000 ) {
		main_loop.command_mode_rcvr++;
	}



	if ( main_loop.laird_bootloader_update_time > 10000000 ) {
		main_loop.laird_bootloader_update_time++;
	}

#ifdef SIGFOX
	main_loop.total_int_count++;
#endif

	main_loop.mag_fcnt_count++;

	main_loop.mag_baseline_report_counter++;


	Mag3110 val;
	uint8_t res = getData( &val );
	if ( res == 0 ) {
		setNewMeas();
		return;
	}


	if ( res == 0 ) {
		err_int = 1;
	}

	pushValue( &mag_queue, val );

	mag_int = NO_INTERRUPT;

}

void restoreFromMemory() {
	//waking from bootloader fw update, restore baseline

	int32_t b_val;
	b_val = nvmReadInteger( BASELINE_VAL );
	data_x.bellum = b_val;
	data_x.b_old = data_x.bellum;

	int32_t mag_val = nvmReadInteger( BASELINE_VAL+4 );
	data_mag.bellum = mag_val;
	data_mag.b_old = mag_val;

	int32_t rot_val = nvmReadInteger( BASELINE_VAL+8 );
	data_rotation.bellum = rot_val;
	data_rotation.b_old = rot_val;

	int32_t total_val = nvmReadInteger( BASELINE_VAL+12 );
	data_total.bellum = total_val;
	data_total.b_old = total_val;


	int32_t total_val1 = nvmReadInteger( BASELINE_VAL+16 );
	data_mag_total.bellum = total_val1;
	data_mag_total.b_old = total_val1;


	if (  nvmReadByte( BASELINE_VAL + 20 ) > 0 && nvmReadByte( BASELINE_VAL + 20 ) < 5 ) {

		data_x.state = 2;
		data_rotation.state = 2;
		data_mag.state = 2;
		data_total.state = 2;


		data_x.main_counter = 151;
		data_rotation.main_counter = 151;
		data_mag.main_counter = 151;
		data_total.main_counter = 151;
		data_mag_total.main_counter = 151;
	}

	send_msg_system( 0, 1, 0, 0, 0, 0, data_x.bellum , data_rotation.bellum , data_mag.bellum, data_total.bellum, data_mag_total.bellum, 0  );


}

void serviceBackup() {
	uint8_t restore = RTC->BKP0R;

	if ( restore == 1 )
		asm( "nop" );

	if ( restore == 16 ) {
		restoreFromMemory();
	}
}

void serviceNRF() {
	if ( nrf_mode == NRF_MODE ) {
				if (  (time_last_update_progress) > (500/magnetic_constant) ) {
					//only send one byte for compatibility with sigfox
					send_msg_bootloader_progress( number_of_distinct_packets );
					time_last_update_progress = 0;
				}
				bootloaderStep();
			}


			if ( nrf_mode == NRF_LAIRD_MODE ) {
				if ( stepLAIRDBootloader() == 1 ) {
					send_msg_laird_bootloader( 1, 0, 0 );

					uint64_t start_time = HAL_GetTick();
					HAL_Delay( 2 );

					while( ( start_time + 180000 ) > HAL_GetTick() && ( start_time ) <= HAL_GetTick()  ) {
						  HAL_IWDG_Refresh( &hiwdg );
					}


					uint32_t ver = getLairdNrfVersion();

					send_msg_laird_version( ver );

					stopNRFMode();



					nrf_mode = old_nrf_mode;
					old_nrf_mode = NO_NRF_MODE;
				}
			}

			if ( nrf_startup == 4 ) {
				initCommandMode();
			}

			if ( nrf_mode == NRF_COMMAND_MODE ) {
				nRFCommandStep();
			}

			if ( nrf_startup == 6 ) {
				stopNRFMode();
				nrf_startup = 1;
			}

			if ( nrf_startup == 10 ) {

				initNRF();
				startNRF( RECEIVE_LAID_BOOTLOADER );

				old_nrf_mode = nrf_mode;
				nrf_mode = NRF_LAIRD_MODE;
				main_loop.laird_bootloader_update_time = 0;
				nrf_startup = 0;


			}

			if ( nrf_startup == 99 ) {

				stopNRFMode();
				nrf_startup = 0;
				nrf_mode = NO_NRF_MODE;
			}

			if ( ( ( nrf_start_time + NRF_COMMAND_TIME ) < HAL_GetTick() && nrf_mode == NRF_COMMAND_MODE ) || nrf_startup == 5 ) {
				stopNRFMode();
				nrf_mode = NO_NRF_MODE;
				if ( nrf_startup == 5 )
					nrf_startup = 0;
				nrf_mag_telemetry = 0;
			}

			if ( nrf_startup == 1 ) {
				time_last_update_progress = 0;
				POWER_20 = 2;

				if ( pht_ons_data == 1 ) {
					nvmWriteOperatingMode( OPERATING_MODE_STORAGE );
				}

				if ( bootloaderInit() == 0 ) {
					send_msg_bootloader( 0, 1, 0, 0 );
					HAL_GPIO_WritePin(BOOST_EN_Port, BOOST_EN_Pin, GPIO_PIN_RESET);
					HAL_GPIO_WritePin(SPI_EN_Port, SPI_EN_Pin, GPIO_PIN_RESET);
					nrf_startup = 0;
					return;
				}
				nrf_startup = 0;
				nrf_mode = NRF_MODE;
			}
}

void serviceMagneticError() {
	resetMagnetometer();
	send_msg_err( 1, 0, 0, 0, 0, 0, 0, 0 );
	err_int = 0;
	main_loop.last_msg_timeout = 0;
}

void serviceMagnetoTelemetry( Mag3110 * val ) {
	 magneticTelemetry( &val );

      initNRF();
      startNRF( RECEIVE_COMMANDS );

	if ( ( startup_time_mag_telemetry + 300000 ) < HAL_GetTick() ) {
		nrf_mag_telemetry = 0;
	}
}

void serviceAMD() {
	uint8_t total_c = 0;

	total_c = data_x.c || data_rotation.c || data_mag.c || data_total.c || data_mag_total.c || data_mag_total.c;

    stepAMD( total_c );

}
void debugPrintoutStep( Mag3110 * val ) {

		//lora_UART_Init();

		char buffer1[ 150 ];
		//sprintf(buffer1, "%d %d %d %d %d %d %d\n", val->x, val->y, val->z, data, vals3, data_total.state,  main_loop.park_state_x  );

		//HAL_UART_Transmit( &huart2, buffer1, strlen( buffer1 ), 1000 );
}


void mode_operational_loop( uint8_t type_init ) {
	uint8_t old_park_state_x = 0;

	mode_operational_init();
	serviceBackup();

	while (1)
	{

			serviceNRF();

			while( isQueueEmpty( &mag_queue ) == 0 ) {

				Mag3110 val = getValue( &mag_queue );


				if ( err_int == 1 && main_loop.last_msg_timeout > 500000 ) {
					serviceMagneticError();
				}

				if ( nrf_mag_telemetry == 1 ) {
					serviceMagnetoTelemetry( &val );
				}

				serviceAMD();
				parkingStep( &val );
				//debugPrintoutStep( &val );

				if ( main_loop.park_state_x == 1 || main_loop.park_state_mag == 1 || main_loop.park_state_rotation == 1 || main_loop.park_state_total == 1 || main_loop.park_state_mag_total == 1 ) {
					main_loop.park_state = 1;
				}else{
					main_loop.park_state = 0;
				}



			    if ( a_buffer_pos_insert == 10 ) {
			        a_buffer[ a_buffer_pos++ ] = val.x;
			        a_buffer[ a_buffer_pos++ ] = val.y;
			        a_buffer[ a_buffer_pos++ ] = val.z;

			        if ( a_buffer_pos > 50 ) {
			            a_buffer_pos = 0;
			        }

			        //a_buffer_pos = a_buffscer_pos + 1;
			        a_buffer_pos_insert = 0;
			    }

			    a_buffer_pos_insert++;



				#ifdef SD_LOGGER
					char data[ 500 ];
					sd_write_to_card( val.x, val.y, val.z, (uint8_t)data_x.state, data_x.b_old, data_x.a, data_x.diff );
				#endif


				if ( old_state == 1 && data_x.state != 1   && data_x.state != 1  && data_rotation.state != 1  && data_mag.state != 1  && data_total.state != 1 && data_mag_total.state != 1 ) {
						//integration done
					for( int x = 0; x < 5; x++ ) {
						blinkLed( 100 );
						HAL_Delay( 300 );
					}


					send_msg_system( 0, 1, 0, 0, 0, 0, data_x.bellum, data_rotation.bellum , data_mag.bellum, data_total.bellum, data_mag_total.bellum, 0 );
					HAL_IWDG_Refresh( &hiwdg );


				}


				if( main_loop.park_state != old_park_state_x  )
				{	//send RF msg if parking state has changed or idle period has been reached
					//send_msg_park_batt( main_loop.park_state, main_loop.batt_state );
						HAL_IWDG_Refresh( &hiwdg );

						uint8_t power = POWER_20;

						uint8_t old_trans_mode = transmisson_mode;
						transmisson_mode = 0;

						if ( nrf_mode == NRF_RESIDENTIAL_MODE &&  main_loop.park_state == 1 ) {
							uint8_t residential_items_counts1 = 0;
							uint32_t start_time = HAL_GetTick();
							uint8_t found = 0;
							residentialParkingInit();
							while( ( ( start_time + residential_mode_time_offset ) > HAL_GetTick() ) && ( start_time <= HAL_GetTick() ) ) {
								found = residentialParkingStep();
								if ( found == 1 ) {
									residential_items_counts1++;
								}

							}
							residentialParkingEnd();
/*
							for( uint8_t x = 0; x < 8; x++ ) {
										if ( deveui_packet_count[ x ] != 0 ) {
											residential_items_counts++;
										}
							}
*/
							if ( residential_items_counts1 > 0 ) {
								  uint8_t old_ack_msgs = ack_msgs;
								  ack_msgs = 1;

								  send_msg_park_residential(main_loop.park_state_x, main_loop.park_state_rotation, main_loop.park_state_mag,  main_loop.park_state_total, main_loop.park_state, deveui_ids, residential_items_counts1, deveui_snr, deveui_packet_count, POWER_20 );

								  ack_msgs = old_ack_msgs;

								 //send_msg_park( main_loop.park_state_x, main_loop.park_state_y, main_loop.park_state_z, main_loop.park_state_x, POWER_20 );
							}



							if ( residential_items_counts1 == 0 ) {
								send_msg_park( main_loop.park_state_x, main_loop.park_state_rotation, main_loop.park_state_mag,  main_loop.park_state_total,  main_loop.park_state, POWER_20 );
							}


						}else{
							send_msg_park( main_loop.park_state_x, main_loop.park_state_rotation, main_loop.park_state_mag,  main_loop.park_state_total, main_loop.park_state, POWER_20 );
						}



						transmisson_mode = old_trans_mode;

						if ( transmisson_mode == 1 )
							send_msg_temp(  main_loop.park_state_x, main_loop.park_state_rotation, main_loop.park_state_mag,  main_loop.park_state_total, main_loop.park_state, power );

						main_loop.idle_cnt = 0;

					}

					//transmisson_mode = 0;
					/*
					if ( main_loop.temp_cnt > 40 ) {
						uint8_t bat_val = getBattEncoded();
						send_msg_batt( bat_val, main_loop.park_state, power );
						main_loop.temp_cnt = 0;
					}else{
						//uint8_t temp_val = getEncodedTemp();
						//temp_val = getEncodedTemp();
						//uint16_t temp_measure = lis3mdl_get_temp();
						//uint8_t te = rint( temp_measure + 20 ) / 5;

						send_msg_temp( 0, main_loop.park_state, power );
						main_loop.temp_cnt++;
					}
					*/

					old_state = data_x.state;
					old_park_state_x =  main_loop.park_state;

			}



			if( main_loop.idle_cnt >= max_idle_periods )
			{

				uint8_t power = POWER_20;

				send_msg_temp( main_loop.park_state_x, main_loop.park_state_rotation, main_loop.park_state_mag,  main_loop.park_state_total, main_loop.park_state, power );

				if ( RTC->BKP0R == 16 ) {
					RTC->BKP0R = 0;
				}

#ifdef RADAR_VERSION
				if ( radar_mode == 1 ) {
					stopNRFMode();
					char radar_sweep_results[ 150 ];

					getRadarData( radar_sweep_results );
					send_msg_radar_sweep_debug( radar_sweep_results );
				}


#endif
				main_loop.batt_cnt += 1;
				main_loop.idle_cnt = 0;
			}
#ifdef SIGFOX
			if ( main_loop.total_int_count >= (MAX_NETWORK_BW_PERIOD/magnetic_constant)  ){
				frame_count_reset();
				send_msg_car_conut( total_car_count );
				total_car_count = 0;
				main_loop.total_int_count = 0;
			}
#endif

			if ( main_loop.laird_bootloader_update_time >=  (1200/magnetic_constant) ) {
/*
				uint16_t prog = getLairdBootloaderPackets();
				send_msg_bootloader_laird_progress( prog );
				main_loop.laird_bootloader_update_time = 0;
				*/
				main_loop.laird_bootloader_update_time = 0;
			}


 			if ( main_loop.mag_reset_offset_count >=  (mag_reset_offset_counter/magnetic_constant) && mag_reset_offset_counter != 1 ) {
				//soft reset without powering the sensor down, use internal coils to reset mag mems

				softReset();
				main_loop.mag_reset_offset_count = 0;
			}



 			if ( main_loop.command_mode_rcvr >=  (nrf_command_rcv_interval/magnetic_constant) && nrf_mode != NRF_MODE && nrf_mode != NRF_LAIRD_MODE  && nrf_mode != 4 ) {
				//soft reset without powering the sensor down, use internal coils to reset mag mems

 			   initNRF();
  			   startNRF( RECEIVE_COMMANDS );

 			   uint64_t start_time = HAL_GetTick();

 			   while( ( start_time + 600 ) > HAL_GetTick() ) {
 				      HAL_IWDG_Refresh( &hiwdg );
 				      uint8_t ret = nRFCommandStep();

 					  if ( ret != 0 ) {

 						  if ( nrf_startup == 0 )
 							  nrf_startup = 4;

 						  send_msg_system_ok_confirmation( ret, 1, 0 );
 					  }

 			   }

 			   stopNRFMode();
 			   HAL_Delay( 100 );
			   main_loop.command_mode_rcvr = 0;
			}


			if ( main_loop.hard_mag_baseline_report_counter >=  (hard_mag_reset_offset_counter/magnetic_constant) && hard_mag_reset_offset_counter != 1  ) {
				//full restart with power down
				resetMagnetometer();
				main_loop.hard_mag_baseline_report_counter = 0;
				//send_msg_baseline_offset(data_x.bellum, MMC34160PJ_offset.x, MMC34160PJ_offset.y, MMC34160PJ_offset.z );
			}

#ifdef MURATA
			if ( imidiate_network_resync == PERFORM_NETWORK_RESYNC  ) {


				lora_send_mac_commands();
				imidiate_network_resync = 0;
				main_loop.mac_command_ressend_counter = 0;
			}
#endif


#ifdef SIGFOX

			if ( main_loop.rcv_counter >= (RECV_TIME_PERIOD/magnetic_constant) ) {

				sigfox_on();
				uint8_t resp = sigfox_recv_msg();
				if ( resp != 1 ) {

					char buf[ 5 ];
					buf[ 0 ] = '6';
					buf[ 1 ] = '1';

					sprintf( buf+2, "%02X", resp );

					radio_send_multibyte_msg( buf, 4,0, 0 );

				}

				sigfox_off();
				main_loop.rcv_counter = 0;
			}

#endif

		while ( mag_int == INTERRUPT ) {
			magneticStep();
		}


		//if ( nrf_mode == NO_NRF_MODE || nrf_mode == NRF_BOUNCE_MODE || nrf_mode == NRF_RESIDENTIAL_MODE || nrf_mode == NRF_LAIRD_MODE )
		//	lowPowerMode();

	}
}

