/*
 * radio.c
 *
 *  Created on: Aug 22, 2017
 *      Author: fiala
 */
#include "radio.h"



void radio_on() {
	RTC->BKP1R = STATE_RADIO_ACTIVE;
	#ifdef MURATA
		lora_on();
	#endif

	#ifdef SIGFOX
		sigfox_on();
	#endif
}

void radio_init_uart() {

	#ifdef MURATA
		lora_UART_Init();
	#endif

	#ifdef SIGFOX
		sigfox_UART_Init();
	#endif

}

void radio_off() {


	#ifdef MURATA

		lora_off();
	#endif

	#ifdef SIGFOX
		sigfox_off();
	#endif

}

uint8_t radio_recv_msg( char * output_buffer, char * hex ) {


#ifdef MURATA
	return lora_recv_msg( output_buffer, hex );
#endif
#ifdef SIGFOX
	sigfox_recv_msg();
#endif
}

uint8_t radio_frame_counter_inc() {

#ifdef MURATA
	//lora_init();
#endif

#ifdef SIGFOX
	sigfox_frame_counter();
#endif


}

void radio_init(void)
{

	#ifdef MURATA
		lora_init();
	#endif

	#ifdef SIGFOX
		sigfox_init();
	#endif
}

void radio_set_power( uint8_t power ) {
	#ifdef MURATA
		if ( power == POWER_20 ) {
			lora_sends( "AT+RFPOWER=1,0\r" );	//14dBm
		}else if( power == POWER_17 ) {
			lora_sends( "AT+RFPOWER=1,1\r" );	//14dBm
		}else if( power == POWER_14 ) {
			lora_sends( "AT+RFPOWER=0,3\r" );	//14dBm
		}
	#endif

	#ifdef SIGFOX
		if ( power == POWER_20 ) {
			sigfox_send( "ATS302=20\r" );
		}else if( power == POWER_17 ) {
			sigfox_send( "ATS302=17\r" );
		}else if( power == POWER_16 ) {
			sigfox_send( "ATS302=16\r" );
		}else if( power == POWER_15 ) {
			sigfox_send( "ATS302=15\r" );
		}else if( power == POWER_14 ) {
			sigfox_send( "ATS302=14\r" );	//14dBm
		}

	#endif
}

uint8_t radio_send_multibyte_msg( char * buffer, int len, uint8_t ignore_downlinks, uint8_t ack_msgss ) {

	#ifdef MURATA
		char msg_radio[130];

		if ( ack_msgss == 1 ) {
			sprintf( msg_radio, "\rAT+CTX %d\r%s\r", len, buffer );
		}else{
			sprintf( msg_radio, "\rAT+UTX %d\r%s\r", len, buffer );

		}

		if ( ignore_downlinks == 1 ) {
			return lora_sends(msg_radio);
		}else{

			return lora_send( msg_radio, ack_msgs );

		}
		//HAL_UART_Transmit( &huart2, "\r", 1, 20 );
		//HAL_UART_Transmit( &huart2, msg_radio, 13, DEFAULT_UART_TIMEOUT );
		// toto nie je finalne riesenie, treba prepracovat!!!!
	#endif
	#ifdef SIGFOX
		char msg_radio[40] = "\rAT$SF=";
		memcpy(msg_radio+7, buffer, len*2);
		msg_radio[ 7 + len*2 ] = '\r';
		msg_radio[ 7 + len*2 + 1 ] = 0;

		sigfox_send( msg_radio );
		//HAL_UART_Transmit( &huart2, "\r", 1, 20 );
		//HAL_UART_Transmit( &huart2, msg_radio, 13, DEFAULT_UART_TIMEOUT );
			// toto nie je finalne riesenie, treba prepracovat!!!!
	#endif
		return 0;
}



uint8_t radio_send_msg( char * hex, uint8_t ack_msgs ) {
	uint8_t resp;
	#ifdef MURATA
		if ( ack_msgs == 1 ) {
			char msg_radio[14] = "\rAT+CTX 1\rxx\r\0";
			memcpy(msg_radio+10, hex, 2);
			resp = lora_send( msg_radio, 1 );
			return resp;
		}else{
			char msg_radio[14] = "\rAT+UTX 1\rxx\r\0";
			memcpy(msg_radio+10, hex, 2);
			lora_send( msg_radio, 0 );

		}


		//magneticDelay( 4000 );
		//HAL_UART_Transmit( &huart2, "\r", 1, 20 );
		//HAL_UART_Transmit( &huart2, msg_radio, 13, DEFAULT_UART_TIMEOUT );
		// toto nie je finalne riesenie, treba prepracovat!!!!
	#endif

#ifdef SIGFOX
	char msg_radio[14] = "\rAT$SF=xx\r\0";
	memcpy(msg_radio+7, hex, 2);
	resp = sigfox_send( msg_radio );
	//HAL_UART_Transmit( &huart2, "\r", 1, 20 );
	//HAL_UART_Transmit( &huart2, msg_radio, 13, DEFAULT_UART_TIMEOUT );
		// toto nie je finalne riesenie, treba prepracovat!!!!
#endif

	return resp;
}

