/*
 * temp.c
 *
 *  Created on: Jun 1, 2017
 *      Author: fiala
 */

#include "temp.h"
#include "battery.h"

#define TEMP130_CAL_ADDR ((uint16_t*) ((uint32_t) 0x1FF8007E))
#define TEMP30_CAL_ADDR ((uint16_t*) ((uint32_t) 0x1FF8007A))
#define VDD_CALIB ((uint16_t) (300))
//#define VDD_APPLI ((uint16_t) (330))
#define VDD_APPLI ((uint16_t) round(vbat*100.0))

extern ADC_HandleTypeDef hadc;
extern struct MM001 radio;
extern UART_HandleTypeDef huart_radio;

//struct temp_t temp;

/* ADC init function */
static void MX_ADC_Init(void)
{

  ADC_ChannelConfTypeDef sConfig;

    /**Configure the global features of the ADC (Clock, Resolution, Data Alignment and number of conversion)
    */
  hadc.Instance = ADC1;
  hadc.Init.OversamplingMode = DISABLE;
  hadc.Init.ClockPrescaler = ADC_CLOCK_SYNC_PCLK_DIV1;
  hadc.Init.Resolution = ADC_RESOLUTION_12B;
  hadc.Init.SamplingTime = ADC_SAMPLETIME_160CYCLES_5;
  hadc.Init.ScanConvMode = ADC_SCAN_DIRECTION_FORWARD;
  hadc.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc.Init.ContinuousConvMode = DISABLE;
  hadc.Init.DiscontinuousConvMode = ENABLE;
  hadc.Init.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_NONE;
  hadc.Init.ExternalTrigConv = ADC_SOFTWARE_START;
  hadc.Init.DMAContinuousRequests = DISABLE;
  hadc.Init.EOCSelection = ADC_EOC_SINGLE_CONV;
  hadc.Init.Overrun = ADC_OVR_DATA_PRESERVED;
  hadc.Init.LowPowerAutoWait = DISABLE;
  hadc.Init.LowPowerFrequencyMode = DISABLE;
  hadc.Init.LowPowerAutoPowerOff = ENABLE;
  if (HAL_ADC_Init(&hadc) != HAL_OK)
  {  }

    /**Configure for the selected ADC regular channel to be converted.
    */
  sConfig.Channel = ADC_CHANNEL_TEMPSENSOR;
  sConfig.Rank = ADC_RANK_CHANNEL_NUMBER;
  if (HAL_ADC_ConfigChannel(&hadc, &sConfig) != HAL_OK)
  {  }

}

float get_temp(void)
{
	uint32_t adcVal = 0;
	adcVal = readADC();

	//calculation
	float vbat = 2.25;

	float temperature = (((float)adcVal * vbat  / 3.0) - (float)*TEMP130_CAL_ADDR);
	temperature = temperature * (float)(130 - 30);
	temperature = temperature / ( ( (float)(*TEMP130_CAL_ADDR ) - ( (float)*TEMP30_CAL_ADDR) ) );
	temperature = temperature + 30.0;
	temperature = temperature + 100.0;

/*
	uint8_t temp_str[20];
	get_dec_str(temp_str,20, (int32_t)(temperature) );
	MM001_sendDataDontWait( &radio, "temperature: ", strlen( "temperature: " ));
	MM001_sendDataDontWait( &radio, temp_str, 20);
	MM001_sendDataDontWait( &radio, " Deg\r\n", strlen( " Deg\r\n" ));
*/
	return temperature;
}

uint8_t getEncodedTemp() {
	 float temp = temp_measure();

	 if ( temp < -20 ) {
		 temp = -20;
	 }

	 if ( temp > 50 ) {
	 		 temp = 50;
	 }

	uint8_t temp_measure = rint( temp + 20 ) / 5;
	return temp_measure;
}

float temp_measure(void)
{
	MX_ADC_Init();
	while(HAL_ADCEx_Calibration_Start(&hadc, ADC_SINGLE_ENDED) != HAL_OK)
		{continue;}

	//calculation
	float temperature = get_temp();

	uint64_t start_time = HAL_GetTick();

	while( HAL_OK != HAL_ADC_Stop(&hadc)  ) {

		if ( ( start_time + 3000 ) < HAL_GetTick() )
			break;

		continue;
	}

	HAL_Delay(200);

	ADC_ChannelConfTypeDef sConfig;
	sConfig.Channel = ADC_CHANNEL_TEMPSENSOR;
	sConfig.Rank = ADC_RANK_NONE;

	start_time = HAL_GetTick();

	while( HAL_OK != HAL_ADC_ConfigChannel(&hadc, &sConfig) ) {
		if ( ( start_time + 3000 ) < HAL_GetTick() )
			break;
	}

	start_time = HAL_GetTick();

	while( HAL_OK !=  HAL_ADC_DeInit(&hadc) ) {
		if ( ( start_time + 3000 ) < HAL_GetTick() )
			break;
	}

	return temperature;
}

#define TEMP_MIN	-30.0
#define TEMP_MAX	+60.0
uint8_t temp_state(float temp)
{
	uint8_t temp_state = (uint8_t)round( 15 * (temp - TEMP_MIN) / (TEMP_MAX - TEMP_MIN) );
	if (temp_state > 15)
		temp_state = 15;
	return temp_state;
}
